<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude">

<xi:include href="master.html" />

<head>
  <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
  <title>Sample Template, for looking at template locals</title>
</head>

<body>
    <h1>All objects from locals():</h1>

    <div py:for="item in sorted(locals()['data'].keys())">
      ${item}: ${repr(locals()['data'][item])}</div>
</body>
</html>
