#flash {
    background-color: yellow;
    border: 4px solid #CCCCCC;
    margin: 10px;
    padding: 5px;
    width: 400px;
    margin-left: auto;
    margin-right: auto;
    font-weight: bold;
}

#acr_slices_menu {
    z-index: 20;
    display: none;
    position: absolute;
    width: 160px;
    background: black;
    border: 2px solid white;
    left: auto;
    top: auto;
    padding: 0;
    margin: 0;
}

.acr_slices_menu_entry {
    display: block;
    float: none;
    margin: none;
    border-left: none;
    border-bottom: 1px solid white;
    text-transform: uppercase;
    padding-left: 4px;
    color: white;
    font-weight: bold;
}

#acr_scripts_menu {
    background:none repeat scroll 0 0 black;
    border:2px solid white;
    display:none;
    left:auto;
    margin:0;
    padding:0;
    position:absolute;
    top:auto;
    min-width:160px;
    z-index: 20px;
}

.acr_scripts_menu_entry {
    display: block;
    float: none;
    margin: none;
    border-left: none;
    border-bottom: 1px solid white;
    text-transform: uppercase;
    padding-left: 4px;
    padding-right: 4px;
    color: white;
    font-weight: bold;
}

.acr_script_del_btn {
    border: 1px solid white;
    margin-right: 2px;
    background-color: gray;
    text-transform: lowercase;
    padding-left: 2px;
}

#acr_admin_menu {
    border: 1px solid gray;
    background-color: black;
    color: lightgray;
    text-align: left;
}

#acr_admin_menu > ul {
    display: block;
    margin: 0;
}

#acr_admin_menu > ul > li {
    list-style: none;
    float: right;
    border-left: 1px solid white;
    padding: 0 0.5em;
    display: block;
    line-height: 200%;
}

#acr_admin_menu a {
    color: white;
    text-decoration: none;
    font-weight: bold;
}

select, textarea, .textfield, .sproxcalendardatetimepicker {
    border: 1px solid #BBBBBB;
}

label {
    margin-left: 5px;
}

.even {
    background-color: lightgray;
}

form table {
    margin: 10px auto;
    width: 100%;
}

form table tr {
    height: 4.5ex;
}

.clearfix {
    clear: both;
}
